//--------------------------------------------------------------
// File     : stm32_ub_mp45dt02.c
// Datum    : 15.06.2013
// Version  : 1.0
// Autor    : UB
// EMail    : mc-4u(@)t-online.de
// Web      : www.mikrocontroller-4u.de
// CPU      : STM32F4
// IDE      : CooCox CoIDE 1.7.0
// Module   : SPI, MISC, STM32_UB_FATFS
// Funktion : MP45DT02 (Digitales Mikrophon)
//            zum aufnehmen von Audio-Daten
//            und speichern als WAV-File auf SD-Karte
//            (Settings = PCM, 16bit, Mono, 16kHz Samplfrq)
//
// LIB      : das File : "libPDMFilter_GCC.a" muss bei CoIDE
//            unter "Configuration/Link" mit dem Button "Add"
//            bei "Linked Libraries" hinzugefgt werden
//
// Hinweis  : benutzt I2S2
//            PB10 = Clock
//            PC3  = Data
//--------------------------------------------------------------


//--------------------------------------------------------------
// Includes
//--------------------------------------------------------------
#include "stm32_ub_mp45dt02.h"



//--------------------------------------------------------------
// Globale Variabeln
//--------------------------------------------------------------
MP45DT02_STATUS_t mp45dt02_status=MP45DT02_NO_INIT;
FIL myWAVFile;
unsigned char rec_wav_buf[REC_WAV_BUF_SIZE];
uint32_t rec_wav_cnt;
uint32_t wav_bytes_cnt;



//--------------------------------------------------------------
// interne Funktionen
//--------------------------------------------------------------
void P_MP45DT02_FillHeader(uint32_t wav_len);
void P_MP45DT02_UpdateHeader(void);
void P_MP45DT02_4Bytes(uint32_t start, uint32_t wert);
void P_MP45DT02_2Bytes(uint32_t start, uint16_t wert);




//--------------------------------------------------------------
// Initialisierung vom MP45DT02
// Return_wert :
//  -> ERROR   , wenn Initialisierung fehlgeschlagen
//  -> SUCCESS , wenn Initialisierung ok war 
//--------------------------------------------------------------
ErrorStatus UB_MP45DT02_Init(void)
{
  ErrorStatus ret_wert=ERROR;

  if(mp45dt02_status==MP45DT02_NO_INIT) {
    
    // Clock der I2S einstellen
    UB_MP45DT02LL_SetI2SClock();

    // init
    UB_MP45DT02LL_Init();
    rec_wav_cnt=0;
    wav_bytes_cnt=0;

    ret_wert=SUCCESS;

    mp45dt02_status=MP45DT02_INIT_OK;
  }


  return(ret_wert);
}


//--------------------------------------------------------------
// legt ein neues WAV-File an und startet die Aufnahme
// Vorsicht : altes File wird gelscht
// Return_wert :
//     MP45DT02_REC_OK , kein Fehler, Aufnahme luft
//  != MP45DT02_REC_OK , Fehler ist aufgetreten
//--------------------------------------------------------------
MP45DT02_ERR_t UB_MP45DT02_RecordStart(const char* name)
{
  MP45DT02_ERR_t ret_wert=MP45DT02_REC_OK;
  FATFS_t check_fat; 

  if(mp45dt02_status==MP45DT02_INIT_OK) {
    // File neu anlegen (alte Daten lschen)
    check_fat=UB_Fatfs_OpenFile(&myWAVFile,name,F_WR_CLEAR); 
    if(check_fat!=FATFS_OK) {
      return(MP45DT02_FILE_ERR);
    }

    // WAV-Header vorbereiten
    P_MP45DT02_FillHeader(0);

    // Counter auf Datenanfang setzen
    rec_wav_cnt=WAV_HEAD_LEN;
    wav_bytes_cnt=0;

    // Aufnahme starten
    UB_MP45DT02LL_RecordStart();

    mp45dt02_status=MP45DT02_RECORD;
    ret_wert=MP45DT02_REC_OK;
  }
  else {
    ret_wert=MP45DT02_INIT_ERR;
  }

  return(ret_wert);
}


//--------------------------------------------------------------
// diese Funktion muss zyklisch aufgerufen werden
//--------------------------------------------------------------
void UB_MP45DT02_RecordLoop(void)
{
  uint8_t check;
  uint32_t n;
  uint16_t pcm_data;
  uint32_t bw_cnt;


  if(mp45dt02_status==MP45DT02_RECORD) {
    check=UB_MP45DT02LL_RecordLoop();

    if(check==1) {
      // wenn lese Puffer voll, kopieren
      for(n=0;n<MP45DT02_DATA_PUFFER_SIZE;n++) {
        // Daten aus Puffer lesen
        pcm_data=*(MP45DT02_Puffer.write_ptr+n);
        // Daten in Wave-Puffer schreiben
        P_MP45DT02_2Bytes(rec_wav_cnt,pcm_data);
        rec_wav_cnt+=2;
        wav_bytes_cnt+=2;
        if(rec_wav_cnt>=REC_WAV_BUF_SIZE) {
          // wenn WAVE-Puffer voll
          rec_wav_cnt=0;
          // Puffer als Block auf SD-Karte speichern
          UB_Fatfs_WriteBlock(&myWAVFile, rec_wav_buf, REC_WAV_BUF_SIZE, &bw_cnt);
        }
      }       
    }    
  }
}


//--------------------------------------------------------------
// stoppt die Aufname und schliet das File
//--------------------------------------------------------------
void UB_MP45DT02_RecordStop(void)
{
  uint32_t bw_cnt;

  if(mp45dt02_status==MP45DT02_RECORD) {
    // Aufnahme stoppen
    UB_MP45DT02LL_RecordStop();
    // letzten Block schreiben
    if(rec_wav_cnt>0) {
      // Puffer als Block auf SD-Karte speichern
      UB_Fatfs_WriteBlock(&myWAVFile, rec_wav_buf, rec_wav_cnt, &bw_cnt);
    }
    // Header im WAV-File updaten
    P_MP45DT02_UpdateHeader();
    // Header als Block auf SD-Karte speichern
    UB_Fatfs_WriteBlock(&myWAVFile, rec_wav_buf, WAV_HEAD_LEN, &bw_cnt);
    // File schlieen
    UB_Fatfs_CloseFile(&myWAVFile);

    mp45dt02_status=MP45DT02_INIT_OK;
  }
}



//--------------------------------------------------------------
// interne Funktion
// erstellt WAV-Header
// Lnge vom Header = 44 Bytes
// wav_len = Anzahl der Daten-Bytes vom WAV
//--------------------------------------------------------------
void P_MP45DT02_FillHeader(uint32_t wav_len)
{
  // RIFF-ID setzen
  rec_wav_buf[0]='R';
  rec_wav_buf[1]='I';
  rec_wav_buf[2]='F';
  rec_wav_buf[3]='F';

  // Filelnge setzen
  P_MP45DT02_4Bytes(4,wav_len+WAV_HEAD_LEN);

  // WAV-ID setzen
  rec_wav_buf[8]='W';
  rec_wav_buf[9]='A';
  rec_wav_buf[10]='V';
  rec_wav_buf[11]='E';

  // fmt-ID setzen
  rec_wav_buf[12]='f';
  rec_wav_buf[13]='m';
  rec_wav_buf[14]='t';
  rec_wav_buf[15]=' ';

  // fmt-Lnge setzen (16d)
  P_MP45DT02_4Bytes(16,16);

  // File-Typ setzen (0x01 = PCM)
  P_MP45DT02_2Bytes(20,01);

  // Anzahl der Kanle setzen (1=Mono)
  P_MP45DT02_2Bytes(22,01);

  // Sample Rate setzen
  P_MP45DT02_4Bytes(24,MP45DT02_SAMPL_FRQ);

  // Byterate setzen (Samplerate*2)
  P_MP45DT02_4Bytes(28,MP45DT02_SAMPL_FRQ*2);

  // Framesize setzen (02d)
  P_MP45DT02_2Bytes(32,2);

  // Bits pro Sample setzen (16d)
  P_MP45DT02_2Bytes(34,16);

  // DATA-ID setzen
  rec_wav_buf[36]='d';
  rec_wav_buf[37]='a';
  rec_wav_buf[38]='t';
  rec_wav_buf[39]='a';  

  // Data lnge setzen
  P_MP45DT02_4Bytes(40,wav_len);  
}


//--------------------------------------------------------------
// interne Funktion
// erneuert den WAV-Header (mit WAV-Laenge)
//--------------------------------------------------------------
void P_MP45DT02_UpdateHeader(void)
{
  // Schreib-Pointer auf Adresse 0 setzen
  f_lseek(&myWAVFile, 0);

  // Header mit WAV-Laenge updaten
  P_MP45DT02_FillHeader(wav_bytes_cnt);
}


//--------------------------------------------------------------
// interne Funktion
// DWORD (4Bytes) in Puffer ab start schreiben
//--------------------------------------------------------------
void P_MP45DT02_4Bytes(uint32_t start, uint32_t wert)
{
  uint8_t cbyte=0;

  cbyte=(wert&0xFF);
  rec_wav_buf[start]=cbyte;
  cbyte=((wert>>8)&0xFF);
  rec_wav_buf[start+1]=cbyte;
  cbyte=((wert>>16)&0xFF);
  rec_wav_buf[start+2]=cbyte;
  cbyte=((wert>>24)&0xFF);
  rec_wav_buf[start+3]=cbyte;
}


//--------------------------------------------------------------
// interne Funktion
// WORD (2Bytes) in Puffer ab start schreiben
//--------------------------------------------------------------
void P_MP45DT02_2Bytes(uint32_t start, uint16_t wert)
{
  uint8_t cbyte=0;

  cbyte=(wert&0xFF);
  rec_wav_buf[start]=cbyte;
  cbyte=((wert>>8)&0xFF);
  rec_wav_buf[start+1]=cbyte;
}
